package cn.com.qmth.examcloud.exchange.outer.api.controller.swufe;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.exchange.outer.api.controller.swufe.bean.SwufeCreateExamDomain;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

/**
 * @author chenken
 * @date 2018年5月7日 下午3:20:33
 * @company QMTH
 * @description 创建考试
 */
@RestController
@RequestMapping("/api/ecs_outlet/exam")
public class SwufeExamController extends ControllerSupport {

	@Autowired
	ExamCloudService examCloudService;

	@ApiOperation(value = "创建考试")
	@PostMapping
	public Map<String, Object> createExam(@RequestBody SwufeCreateExamDomain examReq,
			HttpServletRequest request) {
		tranTimes(examReq);
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		SaveExamReq saveExamReq = new SaveExamReq();
		saveExamReq
				.setBeginTime(DateUtil.parse(examReq.getBeginTime(), DatePatterns.CHINA_DEFAULT));
		saveExamReq.setDuration(examReq.getDuration());
		saveExamReq.setEndTime(DateUtil.parse(examReq.getEndTime(), DatePatterns.CHINA_DEFAULT));
		saveExamReq.setExamType(examReq.getExamType());
		// code不传时取name当code
		if (StringUtils.isNotBlank(examReq.getCode())) {
			saveExamReq.setCode(examReq.getCode());
		} else {
			saveExamReq.setCode(examReq.getName());
		}
		saveExamReq.setName(examReq.getName());
		saveExamReq.setRootOrgId(rootOrgId);
		examCloudService.saveExam(saveExamReq);

		GetExamReq req = new GetExamReq();
		req.setName(examReq.getName());
		req.setRootOrgId(rootOrgId);
		GetExamResp resp = examCloudService.getExam(req);

		Map<String, Object> map = Maps.newHashMap();
		map.put("examId", resp.getId());
		return map;

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudentImportDTO
	 */
	private void tranTimes(SwufeCreateExamDomain exam) {
		String beginTime = exam.getBeginTime();
		if (StringUtils.isNotBlank(beginTime)) {
			beginTime = StringUtils.replace(beginTime, ".000Z", "");
			beginTime = StringUtils.replace(beginTime, "T", " ");
			exam.setBeginTime(beginTime);
		}

		String endTime = exam.getEndTime();
		if (StringUtils.isNotBlank(endTime)) {
			endTime = StringUtils.replace(endTime, ".000Z", "");
			endTime = StringUtils.replace(endTime, "T", " ");
			exam.setEndTime(endTime);
		}
	}

}
