package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.ExamStudentOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.bean.ExamStudent4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.bean.ExamStudentStatus4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.bean.OuterExamStudentBean;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterBatchSaveExamStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterSaveExamStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.BatchOuterSaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterSaveExamStudentResp;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.StatusResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * 考生服务
 *
 * @author WANGWEI
 * @date 2018年7月2日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api(value = "考生服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/examStudent")
public class ExamStudentOuterServiceProvider extends ControllerSupport
		implements
			ExamStudentOuterService {

	private static final long serialVersionUID = -8252740695302763716L;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@ApiOperation(value = "保存考生信息", httpMethod = "POST")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = OuterSaveExamStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("saveExamStudent")
	@Override
	public OuterSaveExamStudentResp saveExamStudent(
			@RequestBody @ApiParam(required = true) OuterSaveExamStudentReq req) {

		if (!getEnterpriseRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		SaveExamStudentReq request = new SaveExamStudentReq();

		request.setCourseCode(req.getCourseCode());
		request.setCourseLevel(req.getCourseLevel());
		request.setCourseName(req.getCourseName());
		request.setExamId(req.getExamId());
		request.setExamName(req.getExamName());
		request.setIdentityNumber(req.getIdentityNumber());
		request.setPaperType(req.getPaperType());
		request.setRootOrgId(req.getRootOrgId());
		request.setStudentCode(req.getStudentCode());
		request.setStudentName(req.getStudentName());
		request.setInfoCollector(req.getInfoCollector());
		request.setSpecialtyName(req.getSpecialtyName());
		request.setExamSite(req.getExamSite());
		request.setGrade(req.getGrade());
		request.setRemark(req.getRemark());

		request.setExt1(req.getExt1());
		request.setExt2(req.getExt2());
		request.setExt3(req.getExt3());
		request.setExt4(req.getExt4());
		request.setExt5(req.getExt5());

		SaveExamStudentResp response = examStudentCloudService.saveExamStudent(request);

		ExamStudentBean sb = response.getExamStudentBean();

		OuterSaveExamStudentResp resp = new OuterSaveExamStudentResp();
		OuterExamStudentBean bean = new OuterExamStudentBean();
		bean.setCourseCode(sb.getCourseCode());
		bean.setCourseLevel(sb.getCourseLevel());
		bean.setCourseName(sb.getCourseName());
		bean.setExamId(sb.getExamId());
		bean.setExamName(sb.getExamName());
		bean.setIdentityNumber(sb.getIdentityNumber());
		bean.setPaperType(sb.getPaperType());
		bean.setRootOrgId(sb.getRootOrgId());
		bean.setStudentCode(sb.getStudentCode());
		bean.setStudentName(sb.getStudentName());

		resp.setExamStudentBean(bean);

		return resp;
	}

	@ApiOperation(value = "批量保存考生信息", httpMethod = "POST")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = BatchOuterSaveExamStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("batchSaveExamStudent")
	@Override
	public BatchOuterSaveExamStudentResp batchSaveExamStudent(
			@RequestBody @ApiParam(required = true) OuterBatchSaveExamStudentReq req) {
		if (!getEnterpriseRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		List<ExamStudent4BatchBean> examStudentList = req.getExamStudentList();

		List<ExamStudentStatus4BatchBean> successList = Lists.newArrayList();
		List<ExamStudentStatus4BatchBean> failureList = Lists.newArrayList();

		for (ExamStudent4BatchBean cur : examStudentList) {
			SaveExamStudentReq request = new SaveExamStudentReq();

			request.setCourseCode(cur.getCourseCode());
			request.setCourseLevel(cur.getCourseLevel());
			request.setCourseName(cur.getCourseName());
			request.setExamId(req.getExamId());
			request.setExamName(req.getExamName());
			request.setIdentityNumber(cur.getIdentityNumber());
			request.setPaperType(cur.getPaperType());
			request.setRootOrgId(req.getRootOrgId());
			request.setStudentCode(cur.getStudentCode());
			request.setStudentName(cur.getStudentName());
			request.setInfoCollector(cur.getInfoCollector());
			request.setSpecialtyName(cur.getSpecialtyName());
			request.setExamSite(cur.getExamSite());
			request.setGrade(cur.getGrade());
			request.setRemark(cur.getRemark());

			request.setExt1(cur.getExt1());
			request.setExt2(cur.getExt2());
			request.setExt3(cur.getExt3());
			request.setExt4(cur.getExt4());
			request.setExt5(cur.getExt5());

			ExamStudentStatus4BatchBean bean = new ExamStudentStatus4BatchBean();
			try {
				SaveExamStudentResp response = examStudentCloudService.saveExamStudent(request);
				ExamStudentBean sb = response.getExamStudentBean();

				bean.setCourseCode(sb.getCourseCode());
				bean.setCourseLevel(sb.getCourseLevel());
				bean.setCourseName(sb.getCourseName());
				bean.setExamId(sb.getExamId());
				bean.setExamName(sb.getExamName());
				bean.setIdentityNumber(sb.getIdentityNumber());
				bean.setStudentCode(sb.getStudentCode());
				bean.setName(sb.getStudentName());
				successList.add(bean);

			} catch (Exception e) {

				bean.setCourseCode(cur.getCourseCode());
				bean.setCourseLevel(cur.getCourseLevel());
				bean.setCourseName(cur.getCourseName());
				bean.setExamId(req.getExamId());
				bean.setExamName(req.getExamName());
				bean.setIdentityNumber(cur.getIdentityNumber());
				bean.setStudentCode(cur.getStudentCode());
				bean.setName(cur.getStudentName());
				failureList.add(bean);

			}

		}

		BatchOuterSaveExamStudentResp resp = new BatchOuterSaveExamStudentResp();
		resp.setSuccessList(successList);
		resp.setFailureList(failureList);
		return resp;
	}

}
