package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.StudentOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.bean.Student4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.bean.StudentStatus4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterBatchSaveStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterSaveStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.BatchOuterSaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterSaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import cn.com.qmth.examcloud.web.config.SystemConfig;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.StatusResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年6月29日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api(value = "学生服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/student")
public class StudentOuterServiceProvider extends ControllerSupport implements StudentOuterService {

	private static final long serialVersionUID = -8252740695302763716L;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	private FaceService faceService;

	@Autowired
	SystemConfig systemConfig;

	@ApiOperation(value = "保存学生信息", httpMethod = "POST")
	@ApiResponses({@ApiResponse(code = 200, message = "成功", response = OuterSaveStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("saveStudent")
	@Override
	public OuterSaveStudentResp saveStudent(
			@RequestBody @ApiParam(required = true) OuterSaveStudentReq req) {
		trim(req);
		Long rootOrgId = req.getRootOrgId();
		if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		SaveStudentReq request = new SaveStudentReq();
		request.setIdentityNumber(req.getIdentityNumber());
		request.setName(req.getName());
		request.setOrgCode(req.getOrgCode());
		request.setOrgId(req.getOrgId());
		request.setOrgName(req.getOrgName());
		request.setPhoneNumber(req.getPhoneNumber());
		request.setRemark(req.getRemark());
		request.setRootOrgId(req.getRootOrgId());
		List<String> studentCodeList = Lists.newArrayList();
		studentCodeList.add(req.getStudentCode());
		request.setStudentCodeList(studentCodeList);

		return processOuterSaveStudentReq(request, req.getPhotoUrl(), req.getOperator());
	}

	private OuterSaveStudentResp processOuterSaveStudentReq(SaveStudentReq saveStudentReq,
			String photoUrl, String operator) {
		// 保存学生
		SaveStudentResp response = studentCloudService.saveStudent(saveStudentReq);
		// 处理照片
		processPhotoUrl(saveStudentReq, photoUrl, operator);

		OuterSaveStudentResp resp = new OuterSaveStudentResp();
		resp.setOrgId(response.getOrgId());
		resp.setOrgName(response.getOrgName());
		resp.setRootOrgId(response.getRootOrgId());
		resp.setStudentId(response.getStudentId());
		return resp;
	}

	/**
	 * 处理照片
	 * 
	 * @param req
	 */
	private void processPhotoUrl(SaveStudentReq request, String photoUrl, String operator) {
		Long rootOrgId = request.getRootOrgId();
		String identityNumber = request.getIdentityNumber();

		if (StringUtils.isNotBlank(photoUrl)) {
			byte[] bs = HttpClientUtil.get(photoUrl);

			int lastIndexOf = photoUrl.lastIndexOf(".");
			if (0 > lastIndexOf) {
				throw new StatusException("100002", "photoPath is not end with photo file suffix.");
			}
			String fileSuffix = photoUrl.substring(lastIndexOf);

			String fileName = System.currentTimeMillis() + fileSuffix;
			File temp = new File(systemConfig.getTempDataDir() + File.separator + "student_photo"
					+ File.separator + fileName);

			try {
				FileUtils.writeByteArrayToFile(temp, bs);
			} catch (IOException e) {
				try {
					FileUtils.forceDelete(temp);
				} catch (IOException ex) {
					// ignore
				}
				throw new StatusException("100003", "文件读写失败", e);
			}

			try {
				faceService.processFace(rootOrgId, identityNumber, fileSuffix, temp, operator);
			} finally {
				try {
					FileUtils.forceDelete(temp);
				} catch (IOException ex) {
					// ignore
				}
			}
		}
	}

	@ApiOperation(value = "批量保存学生信息", httpMethod = "POST")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = BatchOuterSaveStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("batchSaveStudent")
	@Override
	public BatchOuterSaveStudentResp batchSaveStudent(
			@RequestBody @ApiParam(required = true) OuterBatchSaveStudentReq outerBatchSaveStudentReq) {
		if (!getEnterpriseRootOrgId().equals(outerBatchSaveStudentReq.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}
		List<Student4BatchBean> studentList = outerBatchSaveStudentReq.getStudentList();

		List<StudentStatus4BatchBean> successList = Lists.newArrayList();
		List<StudentStatus4BatchBean> failureList = Lists.newArrayList();

		for (Student4BatchBean req : studentList) {
			SaveStudentReq request = new SaveStudentReq();
			request.setIdentityNumber(req.getIdentityNumber());
			request.setName(req.getName());
			request.setOrgCode(req.getOrgCode());
			request.setOrgId(req.getOrgId());
			request.setOrgName(req.getOrgName());
			request.setPhoneNumber(req.getPhoneNumber());
			request.setRemark(req.getRemark());
			request.setRootOrgId(req.getRootOrgId());
			List<String> studentCodeList = Lists.newArrayList();
			studentCodeList.add(req.getStudentCode());
			request.setStudentCodeList(studentCodeList);

			try {
				OuterSaveStudentResp outerSaveStudentResp = processOuterSaveStudentReq(request,
						req.getPhotoUrl(), req.getOperator());
				StudentStatus4BatchBean studentStatus4BatchBeanSuccess = new StudentStatus4BatchBean();
				studentStatus4BatchBeanSuccess.setStudentId(outerSaveStudentResp.getStudentId());
				studentStatus4BatchBeanSuccess.setIdentityNumber(req.getIdentityNumber());
				studentStatus4BatchBeanSuccess.setName(req.getName());
				studentStatus4BatchBeanSuccess.setStudentCode(req.getStudentCode());
				successList.add(studentStatus4BatchBeanSuccess);
			} catch (Exception e) {
				StudentStatus4BatchBean studentStatus4BatchBeanfailure = new StudentStatus4BatchBean();
				studentStatus4BatchBeanfailure.setIdentityNumber(req.getIdentityNumber());
				studentStatus4BatchBeanfailure.setName(req.getName());
				studentStatus4BatchBeanfailure.setStudentCode(req.getStudentCode());
				failureList.add(studentStatus4BatchBeanfailure);
			}
		}
		BatchOuterSaveStudentResp resq = new BatchOuterSaveStudentResp();
		resq.setSuccessList(successList);
		resq.setFailureList(failureList);
		return resq;
	}

}
