package cn.com.qmth.examcloud.exchange.outer.service.impl;

import java.io.File;
import java.net.URI;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.basic.api.FaceCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.FacesetBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetUsableFacesetListReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentFaceReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetUsableFacesetListResp;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class FaceServiceImpl implements FaceService {

	protected ExamCloudLog log = ExamCloudLogFactory.getLog(this.getClass());

	/**
	 * 接口日志
	 */
	private static final ExamCloudLog INTERFACE_LOG = ExamCloudLogFactory
			.getLog("INTERFACE_LOGGER");

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	UpyunService upyunService;

	@Autowired
	FaceCloudService faceCloudService;

	@Value("${$facepp.apiKey}")
	private String apiKey;

	@Value("${$facepp.apiSecret}")
	private String apiSecret;

	@Override
	public void processFace(Long rootOrgId, String identityNumber, String fileSuffix, File file,
			String operator) {

		GetStudentReq req = new GetStudentReq();
		req.setIdentityNumber(identityNumber);
		req.setRootOrgId(rootOrgId);
		GetStudentResp resp = studentCloudService.getStudent(req);
		StudentBean student = resp.getStudentInfo();
		identityNumber = student.getIdentityNumber();
		Long id = student.getId();

		UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
		env.setFileSuffix(fileSuffix);
		env.setRootOrgId(String.valueOf(rootOrgId));
		env.setUserId(String.valueOf(id));

		String photoUrl = upyunService.writeFile("studentBasePhoto", env, file).getUrl();

		String faceToken = detect(photoUrl);

		Map<String, Object> map = addFaceToken2Faceset(rootOrgId, faceToken);

		String facesetToken = (String) map.get("facesetToken");
		Long faceCount = (Long) map.get("faceCount");

		SaveStudentFaceReq request = new SaveStudentFaceReq();
		request.setFacesetToken(facesetToken);
		request.setFaceToken(faceToken);
		request.setOperator(operator);
		String photoName = photoUrl.substring(photoUrl.lastIndexOf("/") + 1);
		request.setPhotoName(photoName);
		request.setRootOrgId(rootOrgId);
		request.setStudentId(student.getId());
		request.setFaceCount(faceCount);

		faceCloudService.saveStudentFace(request);

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @param faceToken
	 * @return
	 */
	private Map<String, Object> addFaceToken2Faceset(Long rootOrgId, String faceToken) {

		Map<String, Object> info = Maps.newHashMap();
		info.put("rootOrgId", rootOrgId);
		info.put("faceToken", faceToken);
		int times = 0;
		while (true) {
			String facesetToken = getFacesetToken(rootOrgId);
			info.put("facesetToken", facesetToken);

			if (times > 5) {
				log.error("添加faceToken至faceSet失败. info=" + JsonUtil.toJson(info));
				return null;
			}

			CloseableHttpClient httpClient = HttpClients.createDefault();
			CloseableHttpResponse httpResponse = null;
			try {
				URI uri = new URIBuilder().setScheme("https").setHost("api-cn.faceplusplus.com")
						.setPath("/facepp/v3/faceset/addface").setParameter("api_key", apiKey)
						.setParameter("api_secret", apiSecret)
						.setParameter("faceset_token", facesetToken)
						.setParameter("face_tokens", faceToken).build();

				INTERFACE_LOG.info("[Face++ Rquest]. url=" + uri.toString());

				HttpPost httpPost = new HttpPost(uri);

				httpResponse = httpClient.execute(httpPost);
				HttpEntity entity = httpResponse.getEntity();
				int statusCode = httpResponse.getStatusLine().getStatusCode();
				String respBody = EntityUtils.toString(entity, "UTF-8");

				INTERFACE_LOG.info(
						"[Face++ Response]. statusCode=" + statusCode + "; respBody=" + respBody);

				if (respBody.contains("QUOTA_EXCEEDED")) {
					throw new StatusException("620006", "faceset is full");
				}

				if (statusCode == HttpStatus.SC_OK) {
					JSONObject jsonObject = JSON.parseObject(respBody);
					int faceAdded = jsonObject.getInteger("face_added");
					int faceCount = jsonObject.getInteger("face_count");
					if (1 == faceAdded) {
						Map<String, Object> ret = Maps.newHashMap();
						ret.put("facesetToken", facesetToken);
						ret.put("faceCount", (long) faceCount);
						return ret;
					}
				} else if (statusCode == HttpStatus.SC_FORBIDDEN) {
					if (respBody.contains("CONCURRENCY_LIMIT_EXCEEDED")) {
						continue;
					}
				}

				times++;
				continue;
			} catch (Exception e) {
				log.error("调用addface接口异常. info=" + JsonUtil.toJson(info), e);
				times++;
				continue;
			} finally {
				HttpClientUtil.close(httpResponse);
			}
		}

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @return
	 */
	private String getFacesetToken(Long rootOrgId) {

		GetUsableFacesetListReq req = new GetUsableFacesetListReq();
		req.setRootOrgId(rootOrgId);
		GetUsableFacesetListResp resp = faceCloudService.getUsableFacesetList(req);
		List<FacesetBean> facesetBeanList = resp.getFacesetBeanList();

		if (CollectionUtils.isEmpty(facesetBeanList)) {
			throw new StatusException("630001", "无可用的faceset");
		}

		FacesetBean facesetBean = facesetBeanList.get(0);
		return facesetBean.getFacesetToken();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param photoUrl
	 * @return
	 */
	private String detect(String photoUrl) {
		int times = 0;
		String faceToken = null;
		while (true) {
			if (times > 5) {
				throw new StatusException("620005", "人脸检测失败");
			}
			CloseableHttpClient httpClient = HttpClients.createDefault();
			CloseableHttpResponse httpResponse = null;
			try {
				URI uri = new URIBuilder().setScheme("https").setHost("api-cn.faceplusplus.com")
						.setPath("/facepp/v3/detect").setParameter("api_key", apiKey)
						.setParameter("api_secret", apiSecret).setParameter("image_url", photoUrl)
						.build();

				INTERFACE_LOG
						.info("[Face++ Rquest]. url=" + uri.toString() + "; photoUrl=" + photoUrl);

				HttpPost httpPost = new HttpPost(uri);

				httpResponse = httpClient.execute(httpPost);
				HttpEntity entity = httpResponse.getEntity();
				int statusCode = httpResponse.getStatusLine().getStatusCode();
				String respBody = EntityUtils.toString(entity, "UTF-8");

				INTERFACE_LOG.info(
						"[Face++ Response]. statusCode=" + statusCode + "; respBody=" + respBody);

				if (statusCode == HttpStatus.SC_OK) {
					JSONObject jsonObject = JSON.parseObject(respBody);
					JSONArray faces = jsonObject.getJSONArray("faces");
					if (1 != faces.size()) {
						throw new StatusException("620009", "检测到多张人脸. photoUrl=" + photoUrl);
					}

					JSONObject face = faces.getJSONObject(0);
					faceToken = face.getString("face_token");
					return faceToken;
				} else if (statusCode == HttpStatus.SC_FORBIDDEN) {
					if (respBody.contains("CONCURRENCY_LIMIT_EXCEEDED")) {
						continue;
					}
				}

				times++;
				continue;
			} catch (Exception e) {
				log.error("调用detect接口异常. photoUrl=" + photoUrl, e);
				times++;
				continue;
			} finally {
				HttpClientUtil.close(httpResponse);
			}
		}

	}

}
