package cn.com.qmth.examcloud.exchange;

import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.web.servlet.MultipartAutoConfiguration;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

import cn.com.qmth.examcloud.web.bootstrap.AppBootstrap;
import cn.com.qmth.examcloud.web.upyun.UpyunSiteManager;

/**
 * Exchange不连接除Redis外的任何数据库. 说明 by wangwei.<br>
 * 
 * inner 提供云平台向南调用的中间件<br>
 * outer 提供第三方接入的中间件<br>
 *
 */

@SpringBootApplication
@Configuration
@EnableEurekaClient
@EnableDiscoveryClient
@ComponentScan(basePackages = {"cn.com.qmth"})
@EnableAutoConfiguration(exclude = {DataSourceAutoConfiguration.class,
		MultipartAutoConfiguration.class})
public class ExchangeApp {

	static {
		String runtimeLevel = System.getProperty("log.commonLevel");
		if (null == runtimeLevel) {
			System.setProperty("log.commonLevel", "DEBUG");
		}
		System.setProperty("hibernate.dialect.storage_engine", "innodb");
	}

	/**
	 * main
	 *
	 * @author WANGWEI
	 * @param args
	 */
	public static void main(String[] args) {
		AppBootstrap.run(ExchangeApp.class, args);

		UpyunSiteManager.init();

		test();
	}

	/**
	 * 测试方法
	 *
	 * @author WANGWEI
	 */
	private static void test() {
		for (int i = 0; i < 100; i++) {
			// Object bean = SpringContextHolder.getBean("DemoBean");
			// bean.doSomeThings();
		}

	}

	@Bean(name = "multipartResolver")
	public MultipartResolver multipartResolver() {
		CommonsMultipartResolver resolver = new CommonsMultipartResolver();
		resolver.setDefaultEncoding("UTF-8");
		resolver.setResolveLazily(true);
		resolver.setMaxInMemorySize(2);
		resolver.setMaxUploadSize(100 * 1024 * 1024);
		return resolver;
	}
}